import math
import numpy as np

G = 6.6742 * 10 ** -5


def distance(s1, s2):
    # note, we're "cheating" a bit here to produce a better animation
    return np.linalg.norm([s1[:2], s2[:2]])


def force(m1, m2, s):
    return (G * m1 * m2) / s ** 2


def angle(vec1, vec2):
    return np.arctan2(vec2[1] - vec1[1], vec2[0] - vec1[0])


def forceVec(vec1, vec2):
    """Calculate the force vector between vec1 and vec2, where vec1 and vec2 are like [x, y, mass]

    The force vector should be of the form [Fx, Fy]
    """
    vec1 = np.array(vec1)
    vec2 = np.array(vec2)
    mass1 = vec1[2]
    mass2 = vec2[2]
    F = force(mass1, mass2, distance(vec1, vec2))
    θ = angle(vec1, vec2)
    return F * np.cos(θ), F * np.sin(θ)  # tuple


def sumOfForces(objects):
    """Given an Nx5 array of objects of the form (x, y, mass, xvel, yvel), return an Nx2 array of the cumulative force acting on each object in x and y"""
    # Non-numpy solution
    all_object_forces = []
    for obj in objects:
        object_forces = []

        # First calculate the forces
        for other_obj in objects:
            if isinstance(obj, np.ndarray) and (obj == other_obj).all():
                # if using numpy
                continue
            elif not isinstance(obj, np.ndarray) and obj == other_obj:
                # if not
                continue
            object_forces.append(forceVec(obj, other_obj))

        # then sum them up:
        sum_x = sum(x[0] for x in object_forces)
        sum_y = sum(x[1] for x in object_forces)
        all_object_forces.append([sum_x, sum_y])

    return all_object_forces


def centerOfMass(objects):
    """Calculate the center of mass of the objects"""
    return (objects[:, 0] * objects[:, 2]).sum() / objects[:, 2].sum(), (
        objects[:, 1] * objects[:, 2]
    ).sum() / objects[:, 2].sum()
